@@ -113,12 +113,14 @@ fitted.PLNfit <- function(object, ...) {
 #' data(trichoptera)
 #' trichoptera <- prepare_data(trichoptera$Abundance, trichoptera$Covariate)
 #' myPLN <- PLN(Abundance ~ 1 + offset(log(Offset)), data = trichoptera)
-#' vcov(myPLN) ## variance-covariance of B
 #' vcov(myPLN, type = "covariance") ## Sigma
 vcov.PLNfit <- function(object, type = c("main", "covariance"), ...) {
   stopifnot(isPLNfit(object))
-  switch(match.arg(type),
-         main       = object$vcov_coef,
+  type <- match.arg(type)
+  if (type == "main" & is.null(attr(object$model_par$B, "vcov_variational")))
+    stop("Variational estimation not available: rerun by setting `variational_var = TRUE` in the control list.")
+  switch(type,
+         main       = attr(object$model_par$B, "vcov_variational"),
          covariance = object$model_par$Sigma)
 }
 
