@@ -9,11 +9,10 @@
 #' @param responses the matrix of responses common to every models
 #' @param covariates the matrix of covariates common to every models
 #' @param offsets the matrix of offsets common to every models
-#' @param control a list for controlling the optimization. See details.
+#' @param weights an optional vector of observation weights to be used in the fitting process.
+#' @param control a list for controlling the optimization.
 #' @param clusters the dimensions of the successively fitted models
 #' @param formula model formula used for fitting, extracted from the formula in the upper-level call
-#' @param control a list for controlling the optimization. See details.
-#' @param xlevels named listed of factor levels included in the models, extracted from the formula in the upper-level call #'
 #' @param cluster the number of clusters of the current model
 #' @param nullModel null model used for approximate R2 computations. Defaults to a GLM model with same design matrix but not latent variable.
 #'
@@ -30,7 +29,6 @@ PLNmixturefit <-
     ## %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     private = list(
       formula    = NA, # the formula call for the model as specified by the user
-      xlevels    = NA, # factor levels present in the original data, useful for predict() methods.
       covariance = NA, # a string describing the covariance model
       comp       = NA, # list of mixture components (PLNfit)
       tau        = NA, # posterior probabilities of cluster belonging
@@ -65,7 +63,7 @@ PLNmixturefit <-
                "gradient"  = Reduce('+', purrr::map2(Tk, Ak, ~ as.numeric(t(diag(.x) %*% X) %*% (.y - Y)))))
         }
 
-        opts <- list("algorithm"="NLOPT_LD_CCSAQ", "xtol_rel"=1.0e-4)
+        opts <- list("algorithm"="NLOPT_LD_CCSAQ", "xtol_rel"=1.0e-6)
         out_optim <- nloptr::nloptr(c(private$Theta), objective_and_gradient,
                                     opts = opts)
         private$Theta <- matrix(out_optim$solution, ncol(X), self$p)
@@ -78,22 +76,25 @@ PLNmixturefit <-
     public  = list(
       #' @description Initialize a [`PLNmixturefit`] model
       #'@param posteriorProb matrix ofposterior probability for cluster belonging
-      initialize = function(responses, covariates, offsets, posteriorProb, formula, xlevels, control) {
+      initialize = function(responses, covariates, offsets, posteriorProb, formula, control) {
         private$tau   <- posteriorProb
         private$comp  <- vector('list', ncol(posteriorProb))
         private$Theta <- matrix(0, ncol(covariates), ncol(responses))
         private$formula <- formula
-        private$xlevels <- xlevels
         private$covariance <- control$covariance
 
         ## Initializing the mixture components (only intercept of group mean)
         mu_k  <- setNames(matrix(1, self$n, ncol = 1), 'Intercept')
         for (k_ in seq.int(ncol(posteriorProb)))
-          private$comp[[k_]] <- PLNfit$new(responses, mu_k, offsets, posteriorProb[, k_], NULL, NULL, control)
+          private$comp[[k_]] <- switch(control$covariance,
+                 "diagonal" = PLNfit_diagonal$new(responses, mu_k, offsets, posteriorProb[, k_], NULL, control),
+                 "full"     = PLNfit$new(responses, mu_k, offsets, posteriorProb[, k_], NULL, control),
+                              PLNfit_spherical$new(responses, mu_k, offsets, posteriorProb[, k_], NULL, control)) # default: spherical
 
       },
       #' @description Optimize a [`PLNmixturefit`] model
-      optimize = function(responses, covariates, offsets, control) {
+      #' @param config a list for controlling the optimization
+      optimize = function(responses, covariates, offsets, config) {
 
         ## The intercept term will serve as the mean in each group/component
         intercept <- matrix(1, nrow(responses), ncol = 1)
@@ -104,13 +105,13 @@ PLNmixturefit <-
         ## ===========================================
         ## INITIALISATION
         cond <- FALSE; iter <- 1
-        objective   <- numeric(control$maxit_out); objective[iter]   <- Inf
-        convergence <- numeric(control$maxit_out); convergence[iter] <- NA
+        objective   <- numeric(config$maxit_out); objective[iter]   <- Inf
+        convergence <- numeric(config$maxit_out); convergence[iter] <- NA
         ## ===========================================
         ## OPTIMISATION
         while (!cond) {
           iter <- iter + 1
-          if (control$trace > 1) cat("", iter)
+          if (config$trace > 1) cat("", iter)
 
           ## ---------------------------------------------------
           ## M - STEP
@@ -121,7 +122,7 @@ PLNmixturefit <-
           }
           ## UPDATE THE MIXTURE MODEL VIA OPTIMIZATION OF PLNmixture
           for (k in seq.int(self$k))
-            self$components[[k]]$optimize(responses, intercept, offsets, private$tau[, k], control)
+            self$components[[k]]$optimize(responses, intercept, offsets, private$tau[, k], config)
 
           ## ---------------------------------------------------
           ## E - STEP
@@ -137,7 +138,7 @@ PLNmixturefit <-
           ## Assess convergence
           objective[iter]   <- -self$loglik
           convergence[iter] <- abs(objective[iter-1] - objective[iter]) /abs(objective[iter])
-          if ((convergence[iter] < control$ftol_out) | (iter >= control$maxit_out)) cond <- TRUE
+          if ((convergence[iter] < config$ftol_out) | (iter >= config$maxit_out)) cond <- TRUE
 
         }
 
@@ -158,17 +159,17 @@ PLNmixturefit <-
       #'  offset and nor covariate effects),
       #'  with weights equal to the posterior probabilities.
       #' @param prior User-specified prior group probabilities in the new data. The default uses a uniform prior.
-      #' @param control a list for controlling the optimization. See [PLN()] for details.
+      #' @param control a list-like structure for controlling the fit. See [PLNmixture_param()] for details.
       #' @param envir Environment in which the prediction is evaluated
       predict = function(newdata,
                          type = c("posterior", "response", "position"),
                          prior = matrix(rep(1/self$k, self$k), nrow(newdata), self$k, byrow = TRUE),
-                         control = list(), envir = parent.frame()) {
+                         control = PLNmixture_param(), envir = parent.frame()) {
 
         type  <- match.arg(type)
 
         ## Extract the model matrices from the new data set with initial formula
-        args <- extract_model(call("PLNmixture", formula = private$formula, data = newdata, xlev = private$xlevels), envir)
+        args <- extract_model(call("PLNmixture", formula = private$formula, data = newdata), envir)
         n_new <- nrow(args$Y)
 
         ## Sanity checks
@@ -183,16 +184,13 @@ PLNmixturefit <-
         tau <- prior
         ## We make a copy of the offset, for accounting for fixed
         ## covariates effects during the alternative algorithm
-        if (ncol(args$X) > 1) {
-          args$O <- args$O + args$X %*% private$Theta
-        }
+        if (ncol(args$X) > 1) args$O <- args$O + args$X %*% private$Theta
 
         ## ===========================================
         ## INITIALISATION
         cond <- FALSE; iter <- 1
-        control <- PLNmixture_param(control, n_new, self$p)
-        objective   <- numeric(control$maxit_out); objective[iter]   <- Inf
-        convergence <- numeric(control$maxit_out); convergence[iter] <- NA
+        objective   <- numeric(control$config_optim$maxit_out); objective[iter]   <- Inf
+        convergence <- numeric(control$config_optim$maxit_out); convergence[iter] <- NA
 
         ## ===========================================
         ## OPTIMISATION
@@ -204,26 +202,26 @@ PLNmixturefit <-
           ## VE step of each component
           ve_step <- list(self$k)
           for (k in seq.int(self$k)) {
-            ve_step[[k]] <- self$components[[k]]$VEstep(intercept, args$O, args$Y, tau[, k], control = control)
+            ve_step[[k]] <- self$components[[k]]$optimize_vestep(intercept, args$O, args$Y, tau[, k])
           }
 
           ## E - STEP
           ## UPDATE THE POSTERIOR PROBABILITIES
           if (self$k > 1) { # only needed when at least 2 components!
             tau <-
-              sapply(ve_step, function(comp) comp$log.lik) %>% # Jik
+              sapply(ve_step, function(comp) comp$Ji) %>% # Jik
               sweep(2, log(colMeans(tau)), "+") %>% # computation in log space
               apply(1, .softmax) %>%        # exponentiation + normalization with soft-max
               t() %>% .check_boundaries()   # bound away probabilities from 0/1
           }
 
           ## Assess convergence
-          J_ik <- sapply(ve_step, function(comp) comp$log.lik)
+          J_ik <- sapply(ve_step, function(comp) comp$Ji)
           J_ik[tau <= .Machine$double.eps] <- 0
           rowSums(tau * J_ik) - rowSums(.xlogx(tau)) + tau %*% log(colMeans(tau))
           objective[iter]   <- -sum(J_ik)
           convergence[iter] <- abs(objective[iter-1] - objective[iter]) /abs(objective[iter])
-          if ((convergence[iter] < control$ftol_out) | (iter >= control$maxit_out)) cond <- TRUE
+          if ((convergence[iter] < control$config_optim$ftol_out) | (iter >= control$config_optim$maxit_out)) cond <- TRUE
 
         }
 
@@ -282,21 +280,20 @@ PLNmixturefit <-
       ## %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       ## Post treatment --------------------
       #' @description Update fields after optimization
-      #' @param weights an optional vector of observation weights to be used in the fitting process.
-      postTreatment = function(responses, covariates, offsets, weights, nullModel) {
+      #' @param config a list for controlling the post-treatment
+      postTreatment = function(responses, covariates, offsets, weights, config, nullModel) {
 
         ## restoring the full design matrix (group means + covariates)
-        mu_k <- setNames(matrix(1, self$n, ncol = 1), 'Intercept')
+        mu_k <- matrix(1, self$n, ncol = 1); colnames(mu_k) <- 'Intercept'
         offsets <- offsets + covariates %*% private$Theta
-
         for (k_ in seq.int(ncol(private$tau)))
           self$components[[k_]]$postTreatment(
             responses,
             mu_k,
             offsets,
             private$tau[,k_],
-            nullModel = nullModel,
-            type = "none"
+            config,
+            nullModel = nullModel
           )
       },
       ## %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
